#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
"""
Functions for performing intercessions
Function name must match intervention type
"""
import logging
import subprocess

import requests

from . import calls

logger = logging.getLogger("mediator")


def force_fail(url: str, wf_id: int):
    """
    Force a specified workflow request to fail

    :return:
    """
    # is HTCondor running this request?
    condor_id = calls.get_condor_id(url, wf_id)
    condor_id = None if condor_id == "None" else condor_id

    if condor_id is not None:
        # evict job from condor, which will also fail the request
        logger.info(f"Evicting request #{wf_id} from HTCondor...")
        if calls.is_docker():
            response = subprocess.run(["condor_rm", f"{condor_id}"], capture_output=True)
        else:
            workflow_container = calls.get_container_id_by_substr("workflow")
            response = subprocess.run(
                ["docker", "exec", "-it", workflow_container, "condor_rm", f"{condor_id}"], capture_output=True
            )

        outerr = response.stderr.decode()
        logger.info(f"{outerr}")
        if "condor_rm:0:There are no jobs in the queue" in outerr:
            # send manual fail
            logger.info(f"No condor job was found for request #{wf_id}. Failing via REST call...")
            workflow_url = f"{url}/workflows/na/requests/{wf_id}/fail"
            logger.info("Sending 'force fail' REST call to workflow service...")
            requests.post(workflow_url)
    else:
        # manually send fail message
        workflow_url = f"{url}/workflows/na/requests/{wf_id}/fail"
        logger.info("Sending 'force fail' REST call to workflow service...")
        requests.post(workflow_url)


def submit(url: str, wf_id: int):
    """
    Manually submit an already existing workflow

    :return:
    """
    wf_name = calls.get_request(url, wf_id)["workflow_name"]
    workflow_url = f"{url}/workflows/{wf_name}/requests/{wf_id}/submit"
    logger.info("Sending 'submit' REST call to workflow service...")
    requests.post(workflow_url)
